from django.conf.urls import include, patterns, url

urlpatterns = patterns('',
    # experimental ajax view
    url(r'/remove/(?P<basketitem_pk>\d+)',
        'basky.views.remove',
         name='remove'),
    url(r'/update/',
        'basky.views.update',
        name='update'),
    url(r'.html',
        'basky.views.basket',
        name='basket'),
)



# -*- coding: utf-8 -*-
import re
from django.conf import settings
from django.contrib import messages
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from basky import settings as basky_settings
from basky import ajax
from basky.models import BasketItem
from basky.config import BasketRegistry


QUANTITY_RE = re.compile('quantity_[\w]{32}')


def remove(request, basketitem_pk):
    """experimental ajax view"""

    # ajax only
    if not request.is_ajax():
        return HttpResponseRedirect(reverse('basky:basket'))

    # get the basket
    basket_name = getattr(
        settings,
        'BASKY_SESSION_KEY_NAME',
        basky_settings.BASKY_SESSION_KEY_NAME)
    basket = request.session[basket_name]

    # get the basket item
    try:
        basketitem = BasketItem.objects.get(pk=basketitem_pk)
    except BasketItem.DoesNotExist:
        response = {
            'success': False,
            'error': 'basketitem does not exist'}
        return ajax.json_response(response)

    # now remove it from the basket
    basket.remove(basketitem)
    response = {'success': True}

    # return the response
    return ajax.json_response(response)


def update(request):
    # next
    url = request.POST.get('next') or reverse('basky:basket')

    # our basketitems
    basketitems = request.session['basket'].basketitem_set.all()

    # get the basket name
    basket_name = getattr(
        settings,
        'BASKY_SESSION_KEY_NAME',
        basky_settings.BASKY_SESSION_KEY_NAME)
    basket = request.session[basket_name]
    if request.POST:
        for key, quantity in request.POST.items():
            if QUANTITY_RE.match(key):

                # turn quantity into an int
                try:
                    quantity = int(quantity)
                except ValueError:
                    continue
                hash = key.replace('quantity_', '')
                items = basketitems.filter(hash=hash)
                try:
                    # get the first basketitem
                    basketitem = items[0]

                    # now update the quantity if > 0
                    if quantity > 0:
                        basket.add(
                            basketitem.content_object,
                            quantity=quantity,
                            append=False,
                            request=request,
                            options=basketitem.options.all())
                    else:
                        basket.remove(basketitem)
                except IndexError:
                    pass
        # removes
        for pk in request.POST.getlist('remove'):
            try:
                basketitem = basket.basketitem_set.get(pk=pk)
                basket.remove(basketitem)
            except BasketItem.DoesNotExist:
                pass

        # add some feedback
        messages.success(
            request,
            'Your basket has been updated')
    return HttpResponseRedirect(url)


def basket(request, **kwargs):

    # get the template
    template_name = kwargs.get('template_name', 'basky/basket.html')
    if request.method == 'POST':

        # get the instance for the form
        try:
            contenttype_pk = request.POST['contenttype_pk']
            object_pk = request.POST['object_pk']
            contenttype = ContentType.objects.get(pk=contenttype_pk)
            instance = contenttype.get_object_for_this_type(pk=object_pk)
            config = BasketRegistry.get(instance)
        except Exception:
            raise Http404('contenttype_pk/object_id in post are incorrect')

        # now get the form for the instance
        form = config.form(request.POST, instance=instance)

        # now process as normal
        if form.is_valid():
            item = form.save()

            # get the basket name
            basket_name = getattr(
                settings,
                'BASKY_SESSION_KEY_NAME',
                basky_settings.BASKY_SESSION_KEY_NAME)

            # now add it to the basket
            basket = request.session[basket_name]

            # prep the options if the form has an options method
            try:
                options = form.options()
            except AttributeError:
                options = None

            # now process
            basketitem = basket.add(
                item,
                quantity=form.cleaned_data['quantity'],
                request=request,
                options=options)

            # wrap up
            if basketitem:
                messages.success(request, 'Basket Updated')
            url = kwargs.get('next') or \
                request.POST.get('next') or \
                reverse('basky:basket')
            return HttpResponseRedirect(url)

    return render_to_response(
        template_name,
        {},
        context_instance=RequestContext(request))



from datetime import datetime, timedelta

from django.core.management.base import BaseCommand

from basky.models import Basket
from basky import settings


class Command(BaseCommand):
    """
    Deletes all the baskets older than BASKY_AGE
    """

    def handle(self, *args, **options):
        # calculate the age
        age = datetime.now() - timedelta(seconds=settings.BASKY_AGE)

        # get the baskets
        baskets = Basket.objects.filter(last_modified__lt=age)
        baskets.delete()



from django.conf import settings
from basky.models import Basket
from basky import settings as basky_settings


class BasketInSessionMiddleware(object):

    def process_request(self, request):
        basket_name = getattr(settings,
                        'BASKY_SESSION_KEY_NAME',
                        basky_settings.BASKY_SESSION_KEY_NAME)
        try:
            b = request.session[basket_name]
        except KeyError:
            b = Basket()
            request.session[basket_name] = b
        # ordering of this is important: save basket, then save session
        b.session_key = request.session.session_key
        # if session key is none
        if b.session_key is None:
            request.session.save()
            b.session_key = request.session.session_key
        # save basket
        b.save()
        # set it back to the session
        request.session[basket_name] = b
        # save session
        request.session.save()



from basky.forms import BasketForm

def register(cls, config=None):
    # create a blank configuration
    if config is None:
        config = BasketConfig()
        for k,v in kwargs.items():
            setattr(config, k, v)
    # generate a key for the class
    key = '%s_%s' % (cls._meta.app_label, cls._meta.module_name)
    # update the config with any kwargs
    # now add it to the registry
    BasketRegistry._registry[key] = config




class BasketRegistry(object):
    _registry = {}
    
    @staticmethod
    def key(model):
        """returns a key for an instance or model class to query registry"""
        return '%s_%s' % (model._meta.app_label, model._meta.module_name)
    
    @staticmethod
    def get(instance):
        """returns the config"""
        key = BasketRegistry.key(instance)
        try:
            config = BasketRegistry._registry[key]
        except KeyError:
                config = BasketConfig
        return config
        

class BasketConfig(object):
    """Basic configuration"""
    name = 'name'
    total = 'total'
    form = BasketForm



# -*- coding: utf-8 -*-
from django import template
from django.contrib.contenttypes.models import ContentType
from basky.config import BasketRegistry
from basky.forms import BasketForm
from django.conf import settings

register = template.Library()


@register.inclusion_tag('basky/_basket.html')
def render_basket(basket):
    return {
        'basket': basket,
        'MEDIA_URL' : settings.MEDIA_URL
    }

@register.inclusion_tag('basky/_basket_form.html')
def basketform(instance):
    config = BasketRegistry.get(instance)
    form = config.form(instance=instance)
    return {
        'form': form
    }



# -*- coding: utf-8 -*-
from django.db import managers



class BasketLineManager(models.Manager):
    
    def grouped(self):
        pass
        # todo basket items grouped into quanities for displaying back to the 
        # the end user


import hashlib
from decimal import Decimal

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.core.cache import cache
from django.db import models
from django.utils.encoding import smart_str

from basky import settings as basky_settings
from basky.signals import (
    pre_remove_from_basket,
    post_remove_from_basket,
    pre_add_to_basket,
    post_add_to_basket,
    basket_is_now_not_empty,
    basket_is_now_empty)


class Basket(models.Model):
    # django model gubbins
    session_key = models.CharField(
        max_length=32,
        db_index=True)
    created = models.DateTimeField(
        auto_now_add=True,
        null=True,
        blank=True)
    last_modified = models.DateTimeField(
        auto_now_add=True,
        null=True,
        blank=True)

    def __unicode__(self):
        return u'Basket: %s' % self.session_key

    def _get_ctype(self, item):
        """return a content type for a given item"""
        return ContentType.objects.get(
            app_label=item._meta.app_label,
            model=item._meta.object_name.lower())

    def _delete_cache(self):
        """delete all of the caches"""
        # delete the cache keys
        cache.delete(self._cache_key('total_items'))
        cache.delete(self._cache_key('total_price'))

    def _cache_key(self, suffix=None):
        """returns the cache_key for a given basket"""
        prefix = getattr(
            settings,
            'BASKY_CACHE_PREFIX',
            basky_settings.BASKY_CACHE_PREFIX)
        if suffix is None:
            key = '%s_%s' % (prefix, self.pk)
        else:
            key = '%s_%s_%s' % (prefix, self.pk, suffix)

        return key

    def total_price(self):
        """return the total price for the items in the basket"""
        # get the key
        key = self._cache_key('total_price')
        cached_total_price = cache.get(key)

        # if there is cache return that
        if cached_total_price is not None:
            return cached_total_price

        # todo : use an aggregate to get this information
        price = Decimal("0.00")
        for basketitem in self.basketitem_set.all():
            price += basketitem.quantity * basketitem.price

        # set the value
        cache.set(key, price)

        return price

    def total_items(self):
        """return the total count for the items in the basket"""
        # get the key
        key = self._cache_key('total_items')
        cached_total_items = cache.get(key)

        # if there is cache return that
        if cached_total_items is not None:
            return cached_total_items

        # no cache, so get total items and set cache
        total_items = 0
        for basketitem in self.basketitem_set.all():
            total_items += basketitem.quantity

        # set the value
        cache.set(key, total_items)

        return total_items

    def add(self, item, **kwargs):
        """
        Add an item to as basket. If quantity is not present in the kwargs
        then a quantity of 1 is assumed. Any negative quantities are
        transformed into positive values.
        """
        # if item is None then content object has been removed loop over
        # basket items & remove any with None as a contentobject
        if item is None:
            for basketitem in self.basketitem_set.all():
                if basketitem.content_object is None:
                    self.remove(basketitem)
            return

        # if quiet was in kwargs we won't send signals
        silent = kwargs.get('silent', False)

        # default return values in case of no signals
        pre_add_return_values = []

        # if quantity wasn't in kwargs, then assume 1
        quantity = kwargs.get('quantity', 1)

        # if quantity was negative, make it positive
        if quantity <= 0:
            quantity = 1

        # make sure it's an int
        try:
            quantity = int(quantity)
        except Exception:
            raise Exception('Quantity is not an integer' % quantity)

        # if price wasn't in the kwargs, then assume the items price
        price = kwargs.get('price', item.total)

        # if description wasn't in the kwargs, then assume the items name
        description = kwargs.get('description', item.name)
        description2 = kwargs.get('description', None)

        # get the options
        options = kwargs.get('options', None)
        if options is not None:
            options_hash = BasketItem.get_options_hash(options)
        else:
            options_hash = None

        # is locked
        locked = kwargs.get('locked', False)

        # are we appending or updating
        append = kwargs.get('append', True)

        # do not add could be in the pre_add signal return, assume False
        do_not_add = False

        # we need the contenttype to do anything
        ctype = self._get_ctype(item)

        # empty?
        basket_empty = True if self.total_items() == 0 else False

        # send the pre_remove_from_basket signal if not in silent mode
        if not silent:
            # make sure that none of the arguments are in the kwargs
            try:
                del(kwargs['quantity'])
            except KeyError:
                pass
            try:
                del(kwargs['item'])
            except KeyError:
                pass
            pre_add_return_values = pre_add_to_basket.send(
                sender=Basket,
                instance=self,
                item=item,
                quantity=quantity,
                **kwargs)

        # now loop over the returns
        for return_value in pre_add_return_values:
            # look for an description
            try:
                description = return_value[1]['description']
            except (TypeError, IndexError, KeyError):
                pass

            # look for an description2
            try:
                description2 = return_value[1]['description2']
            except (TypeError, IndexError, KeyError):
                pass

            # look for an price
            try:
                price = return_value[1]['price']
            except (TypeError, IndexError, KeyError):
                pass

            # now look for a quantity
            try:
                quantity = return_value[1]['quantity']
            except (TypeError, IndexError, KeyError):
                pass

            # now look for a locked
            try:
                locked = return_value[1]['locked']
            except (TypeError, IndexError, KeyError):
                pass

            # now look for a do_not_add
            try:
                do_not_add = return_value[1]['do_not_add']
            except (TypeError, IndexError, KeyError):
                pass

        # if do not add then return False
        if do_not_add:
            return False

        # add to basket - iterate over the quantity
        try:
            basketitem = self.basketitem_set.get(
                description=description,
                description2=description2,
                locked=locked,
                content_type=ctype,
                object_id=item.pk,
                options_hash=options_hash)

            # already have the item in - basket just update the quantity
            if not append:
                basketitem.quantity = quantity
            else:
                basketitem.quantity += quantity

            # delete options that are not the same as the options in item
            try:
                for opt in options:
                    if opt.pk not in [o.pk for o in basketitem.options.all()]:
                        opt.delete()
            except TypeError:
                # options was none - ignore
                pass

            # save it
            basketitem.save()

        except BasketItem.DoesNotExist:
            basketitem = BasketItem()
            basketitem.content_type = ctype
            basketitem.object_id = item.pk
            basketitem.quantity = quantity
            basketitem.price = price
            basketitem.description = description
            basketitem.description2 = description2
            basketitem.locked = locked
            basketitem.basket = self

            if options_hash:
                basketitem.options_hash = options_hash

            # save it
            basketitem.save()
            if options and options_hash:
                for option in options:
                    option.save()
                    basketitem.options.add(option)

        # delete the cache keys
        self._delete_cache()

        # if not in silent send the post add signal
        if not silent:
            post_add_to_basket.send(
                sender=Basket,
                instance=self,
                basketitem=basketitem)

            # if was empty but now not send the basket_is_now_not_empty
            if basket_empty and self.total_items() > 0:
                basket_is_now_not_empty.send(
                    sender=Basket,
                    instance=self)

        # return the basket line we just added
        return basketitem

    def remove(self, basketitem, *args, **kwargs):
        """
        Removes an basketitem to from a basket.
            returns a the item that was deleted
        """
        # if quiet was in kwargs we won't send signals
        silent = kwargs.get('silent', False)

        # pre return incase of no return
        pre_remove_return_values = []

        # do not remove is False by default
        do_not_remove = False

        # empty
        basket_empty = True if self.total_items() == 0 else False

        # send the pre signal
        if not silent:
            pre_remove_return_values = pre_remove_from_basket.send(
                sender=Basket,
                instance=self,
                basketitem=basketitem)

        # now loop over the returns
        for return_value in pre_remove_return_values:
            # now look for a do_not_add
            try:
                do_not_remove = return_value[1]['do_not_remove']
            except (TypeError, IndexError, KeyError):
                pass

        if do_not_remove:
            return False

        try:
            item = basketitem.content_object
            basketitem.options.all().delete()
            basketitem.delete()
        except BasketItem.DoesNotExist:
            pass

        # delete the cache
        self._delete_cache()

        # send the post signal
        if not silent:
            post_remove_from_basket.send(
                sender=Basket,
                instance=self,
                basketitem=basketitem)

            # if was empty but now not send the basket_is_now_not_empty
            if not basket_empty and self.total_items() == 0:
                basket_is_now_empty.send(sender=Basket, instance=self)
        # return the item
        return item

    def empty(self):
        """Empties the basket and destroys the cache"""
        for basketitem in self.basketitem_set.all():
            for option in basketitem.options.all():
                option.delete()
            basketitem.delete()
            self._delete_cache()


class BasketItem(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    quantity = models.PositiveIntegerField(default=1)
    content_object = generic.GenericForeignKey(
        'content_type',
        'object_id')
    description = models.CharField(
        blank=True,
        null=True,
        max_length=255)
    description2 = models.CharField(
        blank=True,
        null=True,
        max_length=255)
    price = models.DecimalField(
        decimal_places=2,
        max_digits=10)
    #
    # TODO: maybe priced options are needed as a foreignkey field?
    #
    options = models.ManyToManyField(
        'BasketItemOption',
        null=True,
        blank=True)
    locked = models.BooleanField(default=False)
    hash = models.CharField(max_length=32)
    options_hash = models.CharField(
        max_length=32,
        blank=True,
        null=True)
    basket = models.ForeignKey(Basket)
    created = models.DateTimeField(
        auto_now_add=True,
        null=True,
        blank=True)

    @property
    def config(self):
        """ returns the config for an object"""
        #
        # TODO: clean this up and fix circular import
        #
        from basky.config import BasketRegistry
        cls = self.content_object.__class__
        key = '%s_%s' % (cls._meta.app_label, cls._meta.module_name)
        try:
            config = BasketRegistry._registry[key]
            return config
        except KeyError:
            pass

    @staticmethod
    def get_options_hash(options):
        if len(options) == 0:
            return None
        signature = ''
        for option in options:
            signature += '%s|%s;' % (
                option.content_type.pk,
                option.object_id)
        m = hashlib.md5()
        m.update(signature)
        return m.hexdigest()

    @property
    def total(self):
        #
        # TODO: maybe cycle through the options
        # here to get the priced options?
        #
        return self.price * self.quantity

    def __unicode__(self):
        return self.description

    def save(self, *args, **kwargs):
        # base64 encode to prevent unicode errors
        m = hashlib.md5()
        m.update('%s' % self.content_type.pk)
        m.update('%s' % self.object_id)
        m.update('%s' % self.price)
        m.update('%s' % smart_str(self.description))
        m.update('%s' % self.options_hash)
        self.hash = m.hexdigest()
        # now save it
        super(BasketItem, self).save(*args, **kwargs)


class BasketItemOption(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    hash = models.CharField(max_length=32)
    content_object = generic.GenericForeignKey(
        'content_type',
        'object_id')
    description = models.CharField(
        blank=True,
        null=True,
        max_length=255)

    @property
    def price(self):
        #
        # TODO: This is where priced options would be implemented
        #
        return Decimal("0.00")

    def __unicode__(self):
        return u'%s' % self.pk

    def save(self, *args, **kwargs):
        m = hashlib.md5()
        m.update('%s' % self.content_type.pk)
        m.update('%s' % self.object_id)
        self.hash = m.hexdigest()

        # now save it
        super(BasketItemOption, self).save(*args, **kwargs)

"""
The models below are not intended for production use but are
for testing purposes - in order for the models to work you'll
need to set BASKY_TEST_MODELS  = True in your settings
"""


class TestComplexProduct(models.Model):
    title = models.CharField(
        max_length=255)
    price = models.DecimalField(
        decimal_places=2,
        max_digits=10)
    sizes = models.ManyToManyField(
        'TestComplexProductOption',
        limit_choices_to={'group__exact': 'size'},
        related_name='size_options',
        blank=True,
        null=True)
    colours = models.ManyToManyField(
        'TestComplexProductOption',
        limit_choices_to={'group__exact': 'colour'},
        related_name='colour_options',
        blank=True,
        null=True)
    description = models.TextField(
        blank=True,
        null=True)
    attribution = models.CharField(
        max_length=255,
        blank=True,
        null=True)

    class Meta:
        managed = basky_settings.BASKY_TEST_MODELS

    def __unicode__(self):
        return u'%s' % self.title

    @property
    def name(self):
        return self.title

    @property
    def total(self):
        return self.price


class TestComplexProductOption(models.Model):
    group = models.CharField(
        max_length=255)
    value = models.CharField(
        max_length=255)

    class Meta:
        managed = basky_settings.BASKY_TEST_MODELS

    def __unicode__(self):
        return u'%s' % self.value


class TestSimpleProduct(models.Model):
    title = models.CharField(
        max_length=255)
    price = models.DecimalField(
        decimal_places=2,
        max_digits=10)
    description = models.TextField(
        blank=True,
        null=True)
    attribution = models.CharField(
        max_length=255,
        blank=True,
        null=True)

    class Meta:
        managed = basky_settings.BASKY_TEST_MODELS

    def __unicode__(self):
        return u'%s' % self.title

    @property
    def name(self):
        return self.title

    @property
    def total(self):
        try:
            for simplediscount in self.simplediscount_set.all():
                self.price *= simplediscount.percentage
        except ValueError:
            pass
        return self.price


class SimpleDiscount(models.Model):
    title = models.CharField(max_length=255)
    simpleproducts = models.ManyToManyField(TestSimpleProduct)
    discount = models.PositiveSmallIntegerField()

    class Meta:
        managed = basky_settings.BASKY_TEST_MODELS

    def __unicode__(self):
        return u'%s' % self.title

    @property
    def percentage(self):
        percentage = Decimal(100 - self.discount)
        percentage = percentage / 100
        return percentage



import json
from django.http import HttpResponse


def error_dict(errors, error_dict=None):
    """returns a dicts of errors for a Django form"""
    if error_dict is None:
        error_dict = {'success': 'false', 'errors': []}
    for k, v in errors.items():
        try:
            error_dict['errors'].append((k, v[0].__unicode__()))
        except AttributeError:
            error_dict['errors'].append((k, v[0]))
    return error_dict


def json_response(response_dict):
    return HttpResponse(
        json.dumps(response_dict, sort_keys=True, indent=2),
        content_type='application/json; charset=UTF-8')



from django.conf import settings
"""
define the name that will be used to access the basket from the session.
"""

BASKY_AGE = getattr(
    settings,
    'BASKY_AGE',
    settings.SESSION_COOKIE_AGE
)


BASKY_SESSION_KEY_NAME = getattr(
    settings,
    'BASKY_SESSION_KEY_NAME',
    'basket')
"""
Defines a maximum amount of items that a basket can hold. The default is
a resonably large number.
"""
# TODO - test this BASKY_MAX_ITEMS
# TODO - make default a decimal and test it
BASKY_MAX_ITEMS = getattr(
    settings,
    'BASKY_MAX_ITEMS',
    9999999)

"""
If set to True the basket will only allow a quantity of one on each item
and will not add another product to the basket if it already exists.
"""
# TODO - test this BASKY_SINGLE_QUANTITY
BASKY_SINGLE_QUANTITY = getattr(
    settings,
    'BASKY_SINGLE_QUANTITY', False)

"""Prefix for the cache"""
BASKY_CACHE_PREFIX = getattr(
    settings,
    'BASKY_CACHE_PREFIX',
    'BASKY')

BASKY_TEST_FIXTURES = getattr(
    settings,
    'BASKY_TEST_FIXTURES',
    [])

BASKY_TEST_MODELS = getattr(
    settings,
    'BASKY_TEST_MODELS',
    False)



import os
from distutils.core import setup
from basky import get_version

root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)

setup(
    name='django-basky',
    version=get_version(),
    license='BSD 3 Clause',
    description='A extensible basket for your django project',
    long_description=open('README.rst').read(),
    author='Jamie Curle',
    author_email='me@jamiecurle.co.uk',
    url='http://django-basky.readthedocs.org',
    package_data={
        'basky': [
            'templates/basky/*.html',
        ],
    },
    packages=[
        'basky',
        'basky.templatetags',
        'basky.management',
        'basky.management.commands',
    ],
    install_requires=[
        'django >= 1.4.10'
    ],
)



# -*- coding: utf-8 -*-
from django.conf import settings
from basky import settings as basky_settings


def basket(request):
    basket_name = getattr(
        settings,
        'BASKY_SESSION_KEY_NAME',
        basky_settings.BASKY_SESSION_KEY_NAME)
    try:
        basket = request.session[basket_name]
    except KeyError:
        basket = None
    return {basket_name: basket}


# -*- coding: utf-8 -*-
from django.dispatch import Signal

"""sent when a request for a products total is requested"""
request_total_for_product = Signal(providing_args=['item'])

"""Sent when a product is removed from basket
basket item is the item that was added"""
pre_remove_from_basket = Signal(providing_args=['basketitem'])
post_remove_from_basket = Signal(providing_args=['basketitem'])

"""sent when a product is added to the basket
basket item is the item that was removed.
    
    .. Note:: that pre_add_to_basket passes the item and quantity and not the 
    basketitem because at the time of sending, it has not yet been
    incorporated into a basketitem
    
receivers can return a dict with {
        'item' : modified_item,
        'quantity' : modified_quantity
}
"""
pre_add_to_basket = Signal(providing_args=['item', 'quantity'])
post_add_to_basket = Signal(providing_args=['basketitem'])

"""Sent when the basket is either now not empty, or now empty, basketitem
is the item that was added/removed"""
basket_is_now_not_empty = Signal()
basket_is_now_empty = Signal()


# -*- coding: utf-8 -*-
from django import forms
from django.contrib.contenttypes.models import ContentType
from models import BasketItem
from exceptions import BasketFormException


class BasketForm(forms.Form):

    contenttype_pk = forms.IntegerField(
        widget=forms.HiddenInput())
    object_pk = forms.IntegerField(
        widget=forms.HiddenInput())
    options = forms.CharField(
        widget=forms.HiddenInput(),
        required=False)
    quantity = forms.IntegerField(
        required=False)
    
    def __init__(self, *args, **kwargs):
        # check for args[0]...
        try:
            post = args[0]
        except IndexError:
            post = None
        # ... or kwargs['item']...
        try:
            instance = kwargs.pop('instance')
        except KeyError:
            instance = None
        # ... if neither present raise BasketFormException
        if not (post or instance):
            msg = 'post data or kwargs[\'instance\'] not present'
            raise BasketFormException(msg)
        # ..now that we know we can continue: get the quantity
        if instance:
            try:
                quantity = kwargs.pop('quantity', 1)
            except KeyError:
                pass
            # get the options
            #
            # TODO - turn this into a 34|32,98|35345 string
            try:
                options = kwargs.pop('options')
            except KeyError:
                options = None
            # workout content type
            ctype = ContentType.objects.get(
                        app_label=instance._meta.app_label,
                        model=instance._meta.object_name.lower())
            # now populate the initial
            initial =  {
                    'contenttype_pk': ctype.pk,
                    'object_pk': instance.pk,
                    'quantity': quantity,
                    'options': options
                }
            # update the kwargs with our initial and let forms do the rest
            kwargs.update({'initial': initial})
            # call super.__init__
        super(BasketForm, self).__init__(*args, **kwargs)

    def save(self):
        ctype = ContentType.objects.get(
                pk=self.cleaned_data['contenttype_pk'])
        item = ctype.model_class().objects.get(
                pk=self.cleaned_data['object_pk'])
        return item



version = (0, 9, 8)


def register(cls, **kwargs):
    from basky.config import BasketRegistry, BasketConfig
    # create a blank configuration
    config = kwargs.get('config', None)
    if config is None:
        config = BasketConfig()
        for k, v in kwargs.items():
            setattr(config, k, v)
    # generate a key for the class
    key = '%s_%s' % (cls._meta.app_label, cls._meta.module_name)
    # update the config with any kwargs
    # now add it to the registry
    BasketRegistry._registry[key] = config


# return a formatted version string
def get_version():
    """returns a pep complient version number"""
    return '.'.join(str(i) for i in version)


# do the autodiscover stuff
def autodiscover():
    """
    Auto discover brazenly copied from django.contrib.admin.
    """
    import copy
    from django.conf import settings
    from django.utils.importlib import import_module
    from basky.config import BasketRegistry

    for app in settings.INSTALLED_APPS:
        try:
            import_module(app)
        except ImportError:
            pass
        # Attempt to import the app's basket module.
        try:
            before_import_registry = copy.copy(BasketRegistry._registry)
            import_module('%s.basket' % app)
        except ImportError:
            pass
        except Exception:
            BasketRegistry._registry = before_import_registry
